@@ -3,6 +3,7 @@
 
 # In[1]:
 
+#ver '0.1.15'
 
 get_ipython().run_line_magic('matplotlib', 'inline')
 import matplotlib.pyplot as plt
@@ -15,9 +16,8 @@ from scipy.sparse import identity
 from scipy.sparse import diags
 
 
-
 class fock_space:
-    def __init__(self, about_excitations, global_exc, statistics):
+    def __init__(self, about_excitations, global_exc, statistics, fermi_sea_modes = None):
 
         self.global_exc = global_exc #internal parameter
         self.statistics = statistics #internal parameter
@@ -46,14 +46,32 @@ class fock_space:
                 self.modes = len(about_excitations)
         #2)
                 self.local_exc = np.array(about_excitations)
+        
+        elif statistics == 'Fermi_sea':
+            if type(about_excitations) is int:
+        #1)
+                self.modes = about_excitations
+        #2)
+                self.local_exc = np.full(self.modes, 1)
+            else:
+        #1)
+                self.modes = len(about_excitations)
+        #2)
+                self.local_exc = np.array(about_excitations)
+            
+            self.fermi_sea_modes = fermi_sea_modes
+                
                 
         else:
-            print('Сhoose the statistics: Bose or Fermi') 
+            print('Сhoose the statistics: Bose, Fermi or Fermi_sea') 
         
         self.local_exc1 = np.array(self.local_exc+1) #internal parameter
         
         #3)
-        self.states_list = list(self.states_generator())
+        if statistics == 'Fermi_sea':
+            self.states_list = list(self.sea_generator())
+        else:
+            self.states_list = list(self.states_generator())
         
         #4)
         self.find_index = {state: idx for idx, state in enumerate(self.states_list)}
@@ -109,7 +127,7 @@ class fock_space:
                 A = coo_matrix((data, (row, col)), shape=(self.dimension , self.dimension ), dtype = complex)
                 self.create.append(A)
        
-        elif statistics == 'Fermi':
+        elif statistics == 'Fermi' or 'Fermi_sea':
         
         #8)
             self.annihilate=[]
@@ -150,9 +168,11 @@ class fock_space:
                         data[i]=(-1)**y*(self.states_list[i][k]+1)**0.5
                 A = coo_matrix((data, (row, col)), shape=(self.dimension , self.dimension ), dtype = complex)
                 self.create.append(A)
-            
+        
+        
+        
         else:
-            print('Сhoose the statistics: Bose or Fermi')
+            print('Сhoose the statistics: Bose, Fermi or Fermi_sea')
                   
                 
     def sigmax(self, i):
@@ -168,6 +188,47 @@ class fock_space:
     def sigmaz(self, i):
 
         return(self.annihilate[i]@self.create[i] - self.create[i]@self.annihilate[i])
+    
+    
+    
+                
+    def ptrace(self, psi, k):
+        psi1 = coo_array(psi)
+        rho = psi1.conj().T @ psi1
+        
+        N = self.global_exc
+        result = np.zeros((N+1,N+1))
+        for p in range(N+1):
+            for q in range(N+1):
+                N1 = N - max(p,q)
+                a = list(self.ngenerator(k, N1, p))
+                b = list(self.ngenerator(k, N1, q))
+                for i in range(len(a)):
+                    result[p,q] += rho[a[i],b[i]]
+        return(result)
+                            
+            
+        
+      
+        
+    def occupations(self,i):
+     
+        if i >= self.dimension :
+            print('the number is out of range')
+            
+        else:
+            return(np.array(self.states_list[i]))
+    
+    
+    def index(self, state):
+       
+        if len(state) != self.modes:
+            print('incorrect size of an array')
+        else:
+            s = tuple(state)
+            return(self.find_index[s])
+    
+
   
         
     def states_generator(self):
@@ -206,45 +267,87 @@ class fock_space:
                     return
                 n -= current_state[j+1] - 1
                 current_state = current_state[:j] + (current_state[j]+1, 0) + current_state[j+2:]
+    
+    
+    def generator_straight(m,N):
+        current_state = (0,)*m
+        n = 0
+        while True:
+            if (n == N):
+                yield current_state
+            j = m - 1
+            current_state = current_state[:j] + (current_state[j]+1,)
+            n += 1
+            while n > N or current_state[j] > 1:
+                j -= 1
+                if j < 0:
+                    return
+                n -= current_state[j+1] - 1
+                current_state = current_state[:j] + (current_state[j]+1, 0) + current_state[j+2:]
+
                 
-    def ptrace(self, psi, k):
-        psi1 = coo_array(psi)
-        rho = psi1.conj().T @ psi1
-        
+    def generator_back(m,N):
+        current_state = (0,)*m
+        n = 0
+        while True:
+            if (n == N):
+                yield current_state
+            j = 0
+            current_state = (current_state[j]+1,) + current_state[j+1:]
+            n += 1
+            while n > N or current_state[j] > 1:
+                j += 1
+                if j >= m:
+                    return
+                n -= current_state[j-1] - 1
+                current_state = current_state[:j-1] + (0, current_state[j]+1) + current_state[j+1:]
+
+
+                
+    def sea_generator(self):
+        m = self.modes
+        m1 = self.fermi_sea_modes
+        m2 = m - m1
         N = self.global_exc
-        result = np.zeros((N+1,N+1))
-        for p in range(N+1):
-            for q in range(N+1):
-                N1 = N - max(p,q)
-                a = list(self.ngenerator(k, N1, p))
-                b = list(self.ngenerator(k, N1, q))
-                for i in range(len(a)):
-                    result[p,q] += rho[a[i],b[i]]
-        return(result)
-                            
-            
         
-      
+        n2=0
+        h2 = (0,)*m2
         
-    def occupations(self,i):
-     
-        if i >= self.dimension :
-            print('the number is out of range')
+        if (N>m1):
+            for n1 in range (min(m1,N),-1,-1):
+                for h1 in generator_straight(m1, n1):
+                    yield (h1+h2)
             
+            
+            for n2 in range (1,min(N-m1,m2)+1):
+                    for n1 in range (m1,-1,-1):
+                        for h2 in generator_back(m2, n2):
+                            for h1 in generator_straight(m1, n1):
+                                yield (h1+h2)
+            
+            if(N-m1<m2):
+                for n2 in range (N-m1+1,N-m1+min(m1,m-N)+1):
+                    for n1 in range (N-n2,-1,-1):
+                        for h2 in generator_back(m2, n2):
+                            for h1 in generator_straight(m1, n1):
+                                yield (h1+h2)
+        
         else:
-            return(np.array(self.states_list[i]))
-    
-    
-    def index(self, state):
-       
-        if len(state) != self.modes:
-            print('incorrect size of an array')
-        else:
-            s = tuple(state)
-            return(self.find_index[s])
+            for n1 in range (m1,-1+ m1-N,-1):
+                for h1 in generator_straight(m1, n1):
+                    yield (h1+h2)
+                    
+            for n2 in range (1,min(m2,N)+1):
+                    for n1 in range (N-n2,-1,-1):
+                        for h2 in generator_back(m2, n2):
+                            for h1 in generator_straight(m1, n1):
+                                yield (h1+h2)
+            
+                
     
 
 
+
         
 class fock_space_kron:
     def __init__(self, f1, f2):
